use crate::{
    auth,
    connection::{handshake::*, ID},
    enc,
};

#[test]
fn test_handshake_dirsync_req() {
    let rand = enc::Random::new();
    let secret = enc::Secret::new_rand(&rand);
    let cipher_send = enc::sym::CipherSend::new(
        enc::sym::CipherKind::XChaCha20Poly1305,
        secret,
        &rand,
    );

    let (_, exchange_key) =
        match enc::asym::KeyExchangeKind::X25519DiffieHellman.new_keypair(&rand)
        {
            Ok(pair) => pair,
            Err(_) => {
                assert!(false, "Can't generate random keypair");
                return;
            }
        };

    let data = dirsync::ReqInner::ClearText(dirsync::ReqData {
        nonce: dirsync::Nonce::new(&rand),
        client_key_id: KeyID(2424),
        id: ID::ID(::core::num::NonZeroU64::new(424242).unwrap()),
        auth: dirsync::AuthInfo {
            user: auth::UserID::new(&rand),
            token: auth::Token::new_anonymous(&rand),
            service_id: auth::SERVICEID_AUTH,
            domain: auth::Domain("example.com".to_owned()),
        },
    });

    let h_req = Handshake::new(HandshakeData::DirSync(dirsync::DirSync::Req(
        dirsync::Req {
            key_id: KeyID(4224),
            exchange: enc::asym::KeyExchangeKind::X25519DiffieHellman,
            hkdf: enc::hkdf::HkdfKind::Sha3,
            cipher: enc::sym::CipherKind::XChaCha20Poly1305,
            exchange_key,
            data,
        },
    )));

    let mut bytes = Vec::<u8>::with_capacity(h_req.len());
    bytes.resize(h_req.len(), 0);
    h_req.serialize(
        cipher_send.kind().nonce_len(),
        cipher_send.kind().tag_len(),
        &mut bytes,
    );

    let deserialized = match Handshake::deserialize(&bytes) {
        Ok(deserialized) => deserialized,
        Err(e) => {
            assert!(false, "{}", e.to_string());
            return;
        }
    };
}
